// Include base class declaration
#include "BasicMeshApp.h"

// Class methods
BasicMeshApp::BasicMeshApp(HINSTANCE hInstance) : D3DApp(hInstance), mFX(0), mTech(0), mVertexLayout(0), mFXMatVar(0)
{
	// Change the window caption
	mMainWndCaption = L"BasicMesh Application";

	// Initialize matricies to identity
	XMMATRIX Ident = XMMatrixIdentity();
	XMStoreFloat4x4(&mView, Ident);
	XMStoreFloat4x4(&mProj, Ident);
	XMStoreFloat4x4(&mWVP, Ident);

	// Initialize camera orientation
	mEye = { 0.0f, 0.0f, 1.0f };
	mAt = { 0.0f, 0.0f, 0.0f };
	mUp = { 0.0f, 1.0f, 0.0f };

};

BasicMeshApp::~BasicMeshApp()
{
	ReleaseCOM(mFX);
	ReleaseCOM(mVertexLayout);
};

bool BasicMeshApp::Init()
{
	// Simply use base class initializer
	if (!D3DApp::Init())
	{
		return false;
	}

	// TODO: Call cube object initialization


	// Create effect and vertex layout
	mFXFileName = L"basicfx.fx";
	buildFX();
	buildVertexLayouts();

	// Initialize view matrix from camera
	mEye = { 5.0f, 5.0f, 5.0f };
	mAt = { 0.0f, 0.0f, 0.0f };
	mUp = { 0.0f, 1.0f, 0.0f };

	XMVECTOR e = XMLoadFloat3(&mEye);
	XMVECTOR a = XMLoadFloat3(&mAt);
	XMVECTOR u = XMLoadFloat3(&mUp);

	XMMATRIX camera = XMMatrixLookAtLH(e,a,u);
	XMStoreFloat4x4(&mView, camera);

	return true;
}

void BasicMeshApp::OnResize()
{
	// Simply use base class resize
	D3DApp::OnResize();

	// TODO: Reset projection matrix
	// Reset (90 degree fov) projection frustum
	XMMATRIX proj = XMMatrixPerspectiveFovLH(0.25f*MathHelper::Pi, AspectRatio(), 1.0f, 1000.0f);
	XMStoreFloat4x4(&mProj, proj);
}

void BasicMeshApp::UpdateScene(float dt)
{};

void BasicMeshApp::DrawScene()
{
	// Check for valid rendering context and swap chain
	assert(md3dImmediateContext);
	assert(mSwapChain);

	// Simply clear render view to blue
	md3dImmediateContext->ClearRenderTargetView(mRenderTargetView, reinterpret_cast<const float*>(&Colors::Blue));

	// Clear depth buffer to 1.0 and stenci buffer to 0
	md3dImmediateContext->ClearDepthStencilView(mDepthStencilView, D3D11_CLEAR_DEPTH | D3D11_CLEAR_STENCIL, 1.0f, 0);

	// TODO: Set input layout to object vertices
	md3dImmediateContext->IASetInputLayout(mVertexLayout);

	// TODO: Compute total modelview-projection matrix
	XMMATRIX v = XMLoadFloat4x4(&mView);
	XMMATRIX p = XMLoadFloat4x4(&mProj);
	XMMATRIX VP = v*p;

	// TODO: Get technique and loop over passes
	D3DX11_TECHNIQUE_DESC td;
	mTech->GetDesc(&td);
	for (UINT p = 0; p < td.Passes; p++)
	{
		// TODO: Render object

	}

	// Swap back buffer
	HR(mSwapChain->Present(0, 0));
};

// Build effect
void BasicMeshApp::buildFX()
{
	// Set shader flags
	DWORD shaderFlags = 0;
#if defined(DEBUG) | defined(_DEBUG)
	shaderFlags |= D3D10_SHADER_DEBUG;
	shaderFlags |= D3D10_SHADER_SKIP_OPTIMIZATION;
#endif

	//TODO: Create shader
	// Load effects file
	ID3D10Blob* shader = 0;
	ID3D10Blob* compilationErrors = 0;
	HRESULT hr = D3DX11CompileFromFile(mFXFileName.c_str(), 0, 0, 0, "fx_5_0", shaderFlags, 0, 0, &shader, &compilationErrors, 0);


	// Output debug info if compilation failed
	if (FAILED(hr))
	{
		// Check for compilation errors or warnings
		if (compilationErrors != 0)
		{
			MessageBoxA(0, (char*)compilationErrors->GetBufferPointer(), 0, 0);
			ReleaseCOM(compilationErrors);
		}
		DXTrace(__FILE__, (DWORD)__LINE__, hr, L"D3DX11CompileFromFile", true);
	}

	// TODO: Create effect from shader
	HR(D3DX11CreateEffectFromMemory(shader->GetBufferPointer(), shader->GetBufferSize(),
		0, md3dDevice, &mFX));
	ReleaseCOM(shader);

	// TODO: Set technique and associate variables
	// Get technique from effect
	mTech = mFX->GetTechniqueByName("BasicTech");

	// Associate modelview-projection shader variable
	mFXMatVar = mFX->GetVariableByName("gWVP")->AsMatrix();
}

// Build vertex layout
void BasicMeshApp::buildVertexLayouts()
{
	// Create vertex layout for pass 0
	D3DX11_PASS_DESC pd;
	mTech->GetPassByIndex(0)->GetDesc(&pd);
	HR(md3dDevice->CreateInputLayout(vertexDesc, 2, pd.pIAInputSignature, pd.IAInputSignatureSize, &mVertexLayout));
}

// Windows main function
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE prevInstance, PSTR cmdLine, int showCmd)
{
	// Add some debug flags if debugging
#if defined(DEBUG) | defined(_DEBUG)
	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
#endif

	// Instantiate application object
	BasicMeshApp theApp(hInstance);

	// Initialize the application
	if (!theApp.Init())
	{
		return 0;
	}

	// Execute the application
	return theApp.Run();
}